use std::hash::Hash;

use ash::vk;

#[derive(Debug, Clone, Copy)]
pub struct Rgba(pub [f32; 4]);

impl std::hash::Hash for Rgba {
    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
        fn hash_f32<H: std::hash::Hasher>(state: &mut H, f: f32) {
            let classify = f.classify();
            match classify {
                std::num::FpCategory::Nan => (classify as u8).hash(state),
                std::num::FpCategory::Infinite | std::num::FpCategory::Zero => {
                    (classify as u8, f.signum() as i8).hash(state)
                }
                std::num::FpCategory::Subnormal
                | std::num::FpCategory::Normal => f.to_bits().hash(state),
            }
        }
        self.0.map(|f| hash_f32(state, f));
    }
}

impl Rgba {
    pub fn new(r: f32, g: f32, b: f32, a: f32) -> Self {
        Self([r, g, b, a])
    }
    pub fn into_u32(&self) -> [u32; 4] {
        self.0.map(|f| (f.clamp(0.0, 1.0) * 255.0) as u32)
    }
    pub fn into_f32(&self) -> [f32; 4] {
        self.0
    }
    pub fn into_snorm(&self) -> [f32; 4] {
        self.0.map(|f| (f - 0.5) * 2.0)
    }
    pub fn into_i32(&self) -> [i32; 4] {
        self.0.map(|f| (f.clamp(0.0, 1.0) * 255.0) as i32 - 128)
    }
}

enum LoadOp {
    Clear(Rgba),
    Load,
    DontCare,
}

enum StoreOp {
    DontCare,
    Store,
}

struct AttachmentInfo {
    size: glam::UVec2,
    format: vk::Format,
    load: LoadOp,
    store: StoreOp,
}

struct Texture {
    texture: vk::Image,
}

pub struct RenderGraph {}
